import { Controller, Get, Post, Body, Patch, Param, Delete, ParseIntPipe, HttpCode, UseGuards, Req, Query, NotFoundException } from '@nestjs/common'
import { ChallengesService } from './challenges.service'
import { CreateChallengeDto } from './dto/create-challenge.dto'
import { UpdateChallengeDto } from './dto/update-challenge.dto'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { ApiBearerAuth, ApiCreatedResponse, ApiForbiddenResponse, ApiNotFoundResponse, ApiOkResponse, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { ChallengeEntity } from './entities/challenge.entity'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'

@Controller('challenges')
export class ChallengesController {
  constructor(private readonly challengesService: ChallengesService) {}

  @Post()
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiCreatedResponse({ type: ChallengeEntity, description: "Objet créé avec succès" })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async create(@Body() createChallengeDto: CreateChallengeDto): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.create(createChallengeDto)
    return new ChallengeEntity(challenge)
  }

  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(ChallengeEntity)
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  async findAll(@Query() queryPagination?: QueryPaginationDto): Promise<PaginateOutputDto<ChallengeEntity>> {
    const [challenges, total] = await this.challengesService.findAll(queryPagination)
    return paginateOutput<ChallengeEntity>(challenges.map(challenge => new ChallengeEntity(challenge)), total, queryPagination)
  }

  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: ChallengeEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async findOne(@Param('id', ParseIntPipe) id: number): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.findOne(id)
    if (!challenge)
      throw new NotFoundException(`Défi inexistant avec l'identifiant ${id}`)
    return new ChallengeEntity(challenge)
  }

  @Patch(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: ChallengeEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async update(@Param('id', ParseIntPipe) id: number, @Body() updateChallengeDto: UpdateChallengeDto) {
    return await this.challengesService.update(id, updateChallengeDto)
  }

  @Delete(':id')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: ChallengeEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async remove(@Param('id', ParseIntPipe) id: number) {
    await this.challengesService.remove(id)
  }

  @Post('/draw-random')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: ChallengeEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async drawRandom(@Req() request: AuthenticatedRequest): Promise<ChallengeEntity> {
    const challenge = await this.challengesService.drawRandom(request.user)
    return new ChallengeEntity(challenge)
  }
}
